//+------------------------------------------------------------------+
//|                    Holt double exponential smoothing - trend.mq4 |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers  5
#property indicator_color1   LimeGreen
#property indicator_color2   LimeGreen
#property indicator_color3   Orange
#property indicator_color4   Orange
#property indicator_color5   DarkGray
#property indicator_width1   2
#property indicator_width3   2
#property indicator_width5   2
#property indicator_level1   0

//
//
//
//
//

extern double             estimatePeriod   = 14.33;
extern double             trendPeriod      = 10;
extern ENUM_APPLIED_PRICE Price            = PRICE_CLOSE;
extern bool               alertsOn         = true;
extern bool               alertsOnCurrent  = true;
extern bool               alertsMessage    = true;
extern bool               alertsSound      = false;
extern bool               alertsNotify     = true;
extern bool               alertsEmail      = true;
extern string             soundFile        = "alert2.wav"; 

extern bool               ShowArrows       = false;
extern string             arrowsIdentifier = "Holt Arrows1";
extern double             arrowsUpperGap   = 0.5;
extern double             arrowsLowerGap   = 0.5;
extern color              arrowsUpColor    = LimeGreen;
extern color              arrowsDnColor    = Red;
extern int                arrowsUpCode     = 241;
extern int                arrowsDnCode     = 242;


//
//
//
//
//

double emaBuffer[];
double emauu[];
double emaud[];
double emadd[];
double emadu[];
double wrkBuffer[];
double slope[];
double alpha;
double gamma;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(7);
   SetIndexBuffer(0,emauu); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,emaud); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,emadd); SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(3,emadu); SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexBuffer(4,wrkBuffer);
   SetIndexBuffer(5,emaBuffer);
   SetIndexBuffer(6,slope);
   
      estimatePeriod = MathMax(estimatePeriod,1);
      trendPeriod    = MathMax(trendPeriod,1);
               alpha = 2.0 /(1.0+estimatePeriod);
               gamma = 2.0 /(1.0+trendPeriod);
      IndicatorShortName("Holt des trend ("+DoubleToStr(estimatePeriod,2)+","+DoubleToStr(trendPeriod,2)+")");
   return(0);
}
int deinit() 
{  
   deleteArrows(); 
return(0); 
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         
   //
   //
   //
   //
   //
            
   for(i=limit; i>=0; i--)
   {
      double price = iMA(NULL,0,1,0,MODE_SMA,Price,i);
         if (i>=(Bars-3))
         {
            emaBuffer[i] = price;
            wrkBuffer[i] = price-emaBuffer[i+1];
            continue;
         }
         
         emaBuffer[i] = alpha*price+(1-alpha)*(emaBuffer[i+1]+wrkBuffer[i+1]);
         wrkBuffer[i] = gamma*(emaBuffer[i]-emaBuffer[i+1]) + (1-gamma)*wrkBuffer[i+1];
         emauu[i] = EMPTY_VALUE;
         emaud[i] = EMPTY_VALUE;
         emadu[i] = EMPTY_VALUE;
         emadd[i] = EMPTY_VALUE;
         slope[i] = slope[i+1];
         if (wrkBuffer[i]>wrkBuffer[i+1]) slope[i] =  1;
         if (wrkBuffer[i]<wrkBuffer[i+1]) slope[i] = -1;
         if (wrkBuffer[i]>0)
            if (slope[i]==1)
                  emauu[i] = wrkBuffer[i];
            else  emaud[i] = wrkBuffer[i];
         if (wrkBuffer[i]<0)
            if (slope[i]==1)
                  emadu[i] = wrkBuffer[i];
            else  emadd[i] = wrkBuffer[i];
            
         //
         //
         //
         //
         //
     
         if (ShowArrows)
         {
            deleteArrow(Time[i]);
            if (slope[i] != slope[i+1])
            {
               if (slope[i] == 1)  drawArrow(i,arrowsUpColor,arrowsUpCode,false);
               if (slope[i] ==-1)  drawArrow(i,arrowsDnColor,arrowsDnCode, true);
            }
          }
   }
   
   //
   //
   //
   //
   //
   
   if (alertsOn)
   {
      if (alertsOnCurrent)
            int whichBar = 0;
      else      whichBar = 1;
      if (slope[whichBar] != slope[whichBar+1])
      if (slope[whichBar] == 1)
            doAlert("sloping up");
      else  doAlert("sloping down");       
   }   
   return(0);
}

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" Holt des trend "+doWhat;
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Holt des trend "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}

